@@ -22,12 +22,10 @@
 #include "mainWindow.h"
 #include "timeLabel.h"
 
-
-#include <cstdlib>
-
 #include <KApplication>
 #include <KCmdLineArgs>
 #include <KCursor>
+#include <KDebug>
 #include <KFileDialog>      //::open()
 #include <KGlobalSettings> //::timerEvent()
 #include <KIO/NetAccess>
@@ -35,9 +33,11 @@
 #include <KMenuBar>
 #include <KSqueezedTextLabel>
 #include <KStatusBar>
+#include <KToggleFullScreenAction>
 #include <KToolBar>
 #include <KWindowSystem>
 #include <KXMLGUIFactory>
+#include <KProtocolInfo>
 
 #include <Phonon/VideoWidget>
 
@@ -56,27 +56,27 @@
 #include <QMouseEvent>
 #include <QObject>
 #include <QTimer>
+#include <QStackedWidget>
 
 #include "actions.h"
 #include "discSelectionDialog.h"
 #include "dbus/playerDbusHandler.h"
 #include "dbus/rootDbusHandler.h"
 #include "dbus/trackListDbusHandler.h"
-#include "debug.h"
 #include "extern.h"         //dialog creation function definitions
-#include "fullScreenAction.h"
 #include "fullScreenToolBarHandler.h"
 #include "messageBox.h"
-#include "mxcl.library.h"
 #include "playDialog.h"  //::play()
 #include "playlistFile.h"
 #include "theStream.h"
 #include "ui_videoSettingsWidget.h"
 #include "videoWindow.h"
+#include "audioView.h"
+#include "loadView.h"
 
 #include <phonon/backendcapabilities.h>
 
-namespace Codeine {
+namespace Dragon {
 
     MainWindow *MainWindow::s_instance = 0;
     /// @see codeine.h
@@ -86,27 +86,41 @@ namespace Codeine {
 
 MainWindow::MainWindow()
         : KXmlGuiWindow()
+        , m_mainView( 0 )
+        , m_audioView(new AudioView(this) )
+        , m_loadView( new LoadView(this) )
+        , m_currentWidget( new QWidget(this) )
         , m_leftDock( 0 )
         , m_positionSlider( 0 )
         , m_volumeSlider( 0 )
         , m_timeLabel( 0 )
         , m_titleLabel( new QLabel( this ) )
         , m_playDialog( 0 )
-        , m_fullScreenAction( 0 )
         , m_stopScreenSaver( 0 )
         , m_stopSleepCookie( -1 )
         , m_toolbarIsHidden(false)
         , m_statusbarIsHidden(false)
 {
-    DEBUG_BLOCK
     s_instance = this;
     setMouseTracking( true );
+
+    m_mainView = new QStackedWidget(this);
+    m_mainView->setMouseTracking( true );
+
     new VideoWindow( this );
     videoWindow()->setMouseTracking( true );
 
     m_positionSlider = videoWindow()->newPositionSlider();
     
-    setCentralWidget( videoWindow() );
+    m_mainView->addWidget(m_loadView);
+    m_mainView->addWidget(m_audioView);
+    m_mainView->addWidget(videoWindow());
+    m_mainView->setCurrentWidget(m_loadView);
+    
+    m_currentWidget = m_loadView;
+
+    setCentralWidget( m_mainView );
+
     setFocusProxy( videoWindow() ); // essential! See VideoWindow::event(), QEvent::FocusOut
 
     m_titleLabel->setMargin( 2 );
@@ -153,7 +167,7 @@ MainWindow::MainWindow()
             make_ratio_action( i18n( "Determine &Automatically" ), "ratio_auto",  Phonon::VideoWidget::AspectRatioAuto );
             make_ratio_action( i18n( "&4:3" ), "ratio_golden", Phonon::VideoWidget::AspectRatio4_3 );
             make_ratio_action( i18n( "Ana&morphic (16:9)" ), "ratio_anamorphic", Phonon::VideoWidget::AspectRatio16_9 );
-	    make_ratio_action( i18n( "&Window Size" ), "ratio_window", Phonon::VideoWidget::AspectRatioWidget );
+            make_ratio_action( i18n( "&Window Size" ), "ratio_window", Phonon::VideoWidget::AspectRatioWidget );
             #undef make_ratio_action
             ac->action( "ratio_auto" )->setChecked( true );
             ac->action( "aspect_ratio_menu" )->menu()->addActions( m_aspectRatios->actions() );
@@ -184,11 +198,17 @@ MainWindow::MainWindow()
 void
 MainWindow::init()
 {
-//     DEBUG_BLOCK
-    connect( engine(), SIGNAL( stateChanged( Phonon::State ) ), this, SLOT( engineStateChanged( Phonon::State ) ) );
+    //connect the stuff in loadView
+    connect( m_loadView, SIGNAL(openDVDPressed()), this, SLOT(playDisc()) );
+    connect( m_loadView, SIGNAL(openFilePressed()), this, SLOT(openFileDialog()) );
+    connect( m_loadView, SIGNAL(loadUrl(KUrl)), this, SLOT(open(KUrl)) );
+
+    //connect the video player
+    connect( engine(), SIGNAL( stateUpdated( Phonon::State, Phonon::State ) ), this, SLOT( engineStateChanged( Phonon::State, Phonon::State ) ) );
     connect( engine(), SIGNAL( currentSourceChanged( Phonon::MediaSource ) ), this, SLOT( engineMediaChanged( Phonon::MediaSource ) ) );
     connect( engine(), SIGNAL( seekableChanged( bool ) ), this, SLOT( engineSeekableChanged( bool ) ) );
     connect( engine(), SIGNAL( metaDataChanged() ), this, SLOT( engineMetaDataChanged() ) );
+    connect( engine(), SIGNAL( hasVideoChanged( bool ) ), this, SLOT( engineHasVideoChanged( bool ) ) );
 
     connect( engine(), SIGNAL( subChannelsChanged( QList< QAction* > ) ), this, SLOT( subChannelsChanged( QList< QAction* > ) ) );
     connect( engine(), SIGNAL( audioChannelsChanged( QList< QAction* > ) ), this, SLOT( audioChannelsChanged( QList< QAction* > ) ) );
@@ -198,7 +218,7 @@ MainWindow::init()
         KMessageBox::error( this, i18n(
             "<qt>xine could not be successfully initialised. Dragon Player will now exit. "
             "You can try to identify what is wrong with your xine installation using the <b>xine-check</b> command at a command-prompt.</qt>") );
-        std::exit( 2 );
+        QApplication::exit( 2 );
     }
 
     //would be dangerous for these to65535 happen before the videoWindow() is initialised
@@ -211,6 +231,7 @@ MainWindow::init()
     statusBar()->addPermanentWidget( m_titleLabel, 100 );
     statusBar()->addPermanentWidget( m_timeLabel );
 
+    Mpris::registerTypes();
     new PlayerDbusHandler( this );
     new RootDbusHandler( this );
     new TrackListDbusHandler( this );
@@ -228,9 +249,6 @@ MainWindow::init()
             args.clear();
             adjustSize(); //will resize us to reflect the videoWindow's sizeHint()
         }
-        else
-            //show the welcome dialog
-            playMedia( true ); // true = show in style of welcome dialog
     }
     else
         //session management must be done after the videoWindow() has been initialised
@@ -239,7 +257,6 @@ MainWindow::init()
 
 MainWindow::~MainWindow()
 {
-    DEBUG_BLOCK
     hide(); //so we appear to have quit, and then sound fades out below
     delete videoWindow(); //fades out sound in dtor
 }
@@ -268,19 +285,25 @@ void MainWindow::wheelEvent (QWheelEvent *event)
 void
 MainWindow::setupActions()
 {
-    DEBUG_BLOCK
 
     KActionCollection * const ac = actionCollection();
 
     KStandardAction::quit( kapp, SLOT( closeAllWindows() ), ac );
-    KStandardAction::open( this, SLOT(playMedia()), ac )->setText( i18n("Play &Media...") );
-    m_fullScreenAction = new FullScreenAction( this, ac );
-    connect( m_fullScreenAction, SIGNAL( toggled( bool ) ), Codeine::mainWindow(), SLOT( setFullScreen( bool ) ) );
+
+    KStandardAction::open( this, SLOT(toggleLoadView()), ac )->setText( i18n("Play &Media...") );
+
+    #define addToAc( X ) ac->addAction( X->objectName(), X );
+
+    KToggleFullScreenAction* toggleFullScreen = new KToggleFullScreenAction( this, ac );
+    toggleFullScreen->setObjectName( "fullscreen" );
+    toggleFullScreen->setShortcut( Qt::Key_F );
+    toggleFullScreen->setAutoRepeat( false );
+    connect( toggleFullScreen, SIGNAL( toggled( bool ) ), Dragon::mainWindow(), SLOT( setFullScreen( bool ) ) );
+    addToAc( toggleFullScreen );
 
     new PlayAction( this, SLOT( play() ), ac );
     new VolumeAction( this, SLOT( toggleVolumeSlider( bool ) ), ac );
 
-    #define addToAc( X ) ac->addAction( X->objectName(), X );
     KAction* playerStop = new KAction( KIcon("media-playback-stop"), i18n("Stop"), ac );
     playerStop->setObjectName( "stop" );
     playerStop->setShortcut( Qt::Key_S );
@@ -305,21 +328,6 @@ MainWindow::setupActions()
     connect( dvdMenu, SIGNAL( triggered() ), engine(), SLOT( toggleDVDMenu() ) );
     addToAc( dvdMenu )
 
-/*
-    KAction* capture = new KAction( KIcon("frame-image"), i18n("Capture Frame"), ac );
-    capture->setObjectName( "frame_image" );
-    capture->setShortcut( Qt::Key_C );
-    connect( capture, SIGNAL( triggered() ), engine(), SLOT( captureFrame() ) );
-    addToAc( capture )
-*/
-/*
-    KToggleAction* OSDShow = new KToggleAction( i18n("Show OSD"), ac );
-    OSDShow->setObjectName( "show_OSD" );
-    OSDShow->setShortcut( Qt::Key_O );
-    connect( OSDShow, SIGNAL( toggled( bool ) ), this, SLOT(  ) );
-    addToAc( OSDShow )
-*/
-
     KAction* positionSlider = new KAction( i18n("Position Slider"), ac );
     positionSlider->setObjectName( "position_slider" );
     positionSlider->setDefaultWidget( m_positionSlider );
@@ -331,39 +339,39 @@ MainWindow::setupActions()
     connect( videoSettings, SIGNAL( toggled( bool ) ), this, SLOT( toggleVideoSettings( bool ) ) );
     addToAc( videoSettings )
 
-    KAction* prev_chapter = new KAction( KIcon("frame-image"), i18n("Previous chapter"), ac );
+    KAction* prev_chapter = new KAction( KIcon("media-skip-backward"), i18n("Previous Chapter"), ac );
     prev_chapter->setObjectName( "prev_chapter" );
     prev_chapter->setShortcut( Qt::Key_Comma );
     connect( prev_chapter, SIGNAL( triggered() ), engine(), SLOT( prevChapter() ) );
     addToAc( prev_chapter )
 
-    KAction* next_chapter = new KAction( KIcon("frame-image"), i18n("Next chapter"), ac );
+    KAction* next_chapter = new KAction( KIcon("media-skip-forward"), i18n("Next Chapter"), ac );
     next_chapter->setObjectName( "next_chapter" );
     next_chapter->setShortcut( Qt::Key_Period );
     connect( next_chapter, SIGNAL( triggered() ), engine(), SLOT( nextChapter() ) );
     addToAc( next_chapter )
 
     // xgettext: no-c-format
-    KAction* tenPercentBack = new KAction( KIcon("frame-image"), i18n("Return 10% back"), ac );
+    KAction* tenPercentBack = new KAction( KIcon("media-seek-backward"), i18n("Return 10% Back"), ac );
     tenPercentBack->setObjectName( "ten_percent_back" );
     tenPercentBack->setShortcut( Qt::Key_PageUp );
     connect( tenPercentBack, SIGNAL( triggered() ), engine(), SLOT( tenPercentBack() ) );
     addToAc( tenPercentBack )
 
     // xgettext: no-c-format
-    KAction* tenPercentForward = new KAction( KIcon("frame-image"), i18n("Go 10% forward"), ac );
+    KAction* tenPercentForward = new KAction( KIcon("media-seek-forward"), i18n("Go 10% Forward"), ac );
     tenPercentForward->setObjectName( "ten_percent_forward" );
     tenPercentForward->setShortcut( Qt::Key_PageDown );
     connect( tenPercentForward, SIGNAL( triggered() ), engine(), SLOT( tenPercentForward() ) );
     addToAc( tenPercentForward )
 
-    KAction* tenSecondsBack = new KAction( KIcon("frame-image"), i18n("Return 10 seconds back"), ac );
+    KAction* tenSecondsBack = new KAction( KIcon("media-seek-backward"), i18n("Return 10 Seconds Back"), ac );
     tenSecondsBack->setObjectName( "ten_seconds_back" );
     tenSecondsBack->setShortcut( Qt::Key_Minus );
     connect( tenSecondsBack, SIGNAL( triggered() ), engine(), SLOT( tenSecondsBack() ) );
     addToAc( tenSecondsBack )
 
-    KAction* tenSecondsForward = new KAction( KIcon("frame-image"), i18n("Go 10 seconds forward"), ac );
+    KAction* tenSecondsForward = new KAction( KIcon("media-seek-forward"), i18n("Go 10 Seconds Forward"), ac );
     tenSecondsForward->setObjectName( "ten_seconds_forward" );
     tenSecondsForward->setShortcut( Qt::Key_Plus );
     connect( tenSecondsForward, SIGNAL( triggered() ), engine(), SLOT( tenSecondsForward() ) );
@@ -391,6 +399,7 @@ MainWindow::toggleVideoSettings( bool show )
         foreach( QSlider* slider, m_sliders )
              connect( slider, SIGNAL( valueChanged( int ) ), engine(), SLOT( settingChanged( int ) ) );
         
+        connect( ui.defaultsButton, SIGNAL( clicked( bool ) ), this, SLOT( restoreDefaultVideoSettings() ) );
         connect( ui.closeButton, SIGNAL( clicked( bool ) ), action( "video_settings" ), SLOT( setChecked( bool ) ) );
         connect( ui.closeButton, SIGNAL( clicked( bool ) ), m_leftDock, SLOT( deleteLater() ) );
     }
@@ -408,6 +417,37 @@ MainWindow::restoreDefaultVideoSettings()
         slider->setValue(0);
 }
 
+void
+MainWindow::toggleLoadView()
+{
+  if( engine()->state() == Phonon::PlayingState && TheStream::hasVideo() )
+  {
+    engine()->playPause();
+  }
+    if( m_mainView->currentWidget() == m_loadView )
+    {
+      if( m_mainView->indexOf(m_currentWidget) == -1 )
+      {
+        m_mainView->addWidget(m_currentWidget);
+        engine()->playPause();
+      }
+      m_mainView->setCurrentWidget(m_currentWidget);
+      engine()->isPreview(false);
+    }
+    else if( m_currentWidget != m_audioView )
+    {
+      kDebug() << "setting Thumbnail for video Widget";
+      m_mainView->setCurrentWidget(m_loadView);
+      m_mainView->removeWidget(m_currentWidget);
+      engine()->isPreview(true);
+      m_loadView->setThumbnail(m_currentWidget);
+    }
+    else 
+    {
+      m_mainView->setCurrentWidget(m_loadView);
+    }
+}
+
 void
 MainWindow::toggleVolumeSlider( bool show )
 {
@@ -461,24 +501,27 @@ MainWindow::updateSliders()
 void
 MainWindow::engineMessage( const QString &message )
 {
-    DEBUG_BLOCK
     statusBar()->showMessage( message, 3500 );
 }
 
 bool
 MainWindow::open( const KUrl &url )
 {
-    DEBUG_BLOCK
-    debug() << url;
+    kDebug() << url;
 
     if( load( url ) ) {
         const int offset = TheStream::hasProfile()
                 // adjust offset if we have session history for this video
                 ? TheStream::profile().readEntry<int>( "Position", 0 )
                 : 0;
-        debug() << "Initial offset is "<< offset;
+        kDebug() << "Initial offset is "<< offset;
         engine()->loadSettings();
         updateSliders();
+        if( TheStream::hasVideo() )
+          m_currentWidget = engine();
+        else
+          m_currentWidget = m_audioView;
+        m_mainView->setCurrentWidget(m_currentWidget);
         return engine()->play( offset );
     }
 
@@ -508,18 +551,21 @@ MainWindow::load( const KUrl &url )
         }
     }
 
-    if (url.protocol() == "media") {
+    // local protocols like nepomuksearch:/ are not supported by xine
+    // check if an UDS_LOCAL_PATH is defined.
+    if (KProtocolInfo::protocolClass(url.protocol()) == QLatin1String(":local")) {
         //#define UDS_LOCAL_PATH (72 | KIO::UDS_STRING)
         KIO::UDSEntry e;
-        if (!KIO::NetAccess::stat( url, e, 0 ))
-            MessageBox::sorry( i18n("There was an internal error with the media slave...") );
-        else {
+        if (KIO::NetAccess::stat( url, e, 0 )) {
             QString path = e.stringValue( KIO::UDSEntry::UDS_LOCAL_PATH );
             if( !path.isEmpty() )
                 return engine()->load( KUrl( path ) );
         }
     }
 
+    if( m_mainView->indexOf(engine()) == -1 )
+      toggleLoadView();
+
     //let xine handle invalid, etc, KUrlS
     //TODO it handles non-existing files with bad error message
     return engine()->load( url );
@@ -534,9 +580,16 @@ MainWindow::play()
         break;
     case Phonon::PausedState:
         engine()->resume();
+        if( m_mainView->currentWidget() == m_loadView )
+          toggleLoadView();
         break;
     case Phonon::StoppedState:
+        if( TheStream::hasVideo() )
+          m_currentWidget = engine();
+        else
+          m_currentWidget = m_audioView;
         engine()->play();
+        m_mainView->setCurrentWidget(engine());
         break;
     default:
         break;
@@ -544,59 +597,29 @@ MainWindow::play()
 }
 
 void
-MainWindow::playMedia( bool show_welcome_dialog )
+MainWindow::openFileDialog()
 {
-    DEBUG_BLOCK
-    if( ! m_playDialog )
-    {
-        m_playDialog = new Codeine::PlayDialog( this, show_welcome_dialog );
-        m_playDialog->show();
-    }
-    else
-    {
-        debug() << "playdialog not null";
-    }
-}
-
-void
-MainWindow::playDialogResult( int result )
-{
-    DEBUG_BLOCK
-    switch( result ) {
-    case PlayDialog::FILE: {
-        QStringList mimeFilter=Phonon::BackendCapabilities::availableMimeTypes();
+       QStringList mimeFilter=Phonon::BackendCapabilities::availableMimeTypes();
         //temporary fixes for MimeTypes that Xine does support but it doesn't return - this is a Xine bug.
         mimeFilter << "audio/x-flac";
         mimeFilter << "video/mp4";
         mimeFilter << "application/x-cd-image"; // added for *.iso images
 
-        const KUrl url = KFileDialog::getOpenUrl( KUrl("kfiledialog:///dragonplayer"),mimeFilter.join(" "), this, i18n("Select A File To Play") );
+        const KUrl url = KFileDialog::getOpenUrl( KUrl("kfiledialog:///dragonplayer"),mimeFilter.join(" "), this, i18n("Select File to Play") );
         if( url.isEmpty() )
         {
-             debug() << "URL empty in MainWindow::playDialogResult()";
+            kDebug() << "URL empty in MainWindow::playDialogResult()";
             return;
         }
         else
-            this->open( url );
-        } break;
-    case PlayDialog::RECENT_FILE:
-       
-        break;
-    case PlayDialog::VCD:
-        this->open( KUrl( "vcd://" ) ); // one / is not enough
-        break;
-    case PlayDialog::DVD:
-        playDisc();
-        break;
-    }
-    m_playDialog->deleteLater();
-    m_playDialog = 0;
+        {
+            load( url );
+        }
 }
 
 void
 MainWindow::playDisc()
 {
-DEBUG_BLOCK
     QList< Solid::Device > playableDiscs;
     {
         QList< Solid::Device > deviceList = Solid::Device::listFromType( Solid::DeviceInterface::OpticalDisc );
@@ -616,16 +639,16 @@ DEBUG_BLOCK
     {
         if( playableDiscs.size() > 1 ) //more than one disc, show user a selection box
         {
-            debug() << "> 1 possible discs, showing dialog";
+            kDebug() << "> 1 possible discs, showing dialog";
             new DiscSelectionDialog( this, playableDiscs );
         }
         else //only one optical disc inserted, play whatever it is
         {
-            debug() << "playing disc", engine()->playDisc( playableDiscs.first() );
+            kDebug() << "playing disc", engine()->playDisc( playableDiscs.first() );
         }
     }
     else
-        engine()->playDvd(), debug() << "no disc in drive or Solid isn't working";
+        engine()->playDvd(); kDebug() << "no disc in drive or Solid isn't working";
 
 }
 
@@ -640,8 +663,7 @@ MainWindow::openRecentFile( const KUrl& url )
 void
 MainWindow::setFullScreen( bool isFullScreen )
 {
-    DEBUG_BLOCK
-    debug() << "Setting full screen to " << isFullScreen;
+    kDebug() << "Setting full screen to " << isFullScreen;
     mainWindow()->setWindowState( (isFullScreen ? Qt::WindowFullScreen : Qt::WindowNoState ));
     static FullScreenToolBarHandler *s_handler;
 
@@ -682,12 +704,11 @@ MainWindow::showVolume( bool visible)
 void
 MainWindow::aboutToShowMenu()
 {
-    DEBUG_BLOCK
     TheStream::aspectRatioAction()->setChecked( true );
     {
         int subId = TheStream::subtitleChannel();
         QList< QAction* > subs = action("subtitle_channels_menu")->menu()->actions();
-        debug() << "subtitle #" << subId << " is going to be checked";
+        kDebug() << "subtitle #" << subId << " is going to be checked";
         foreach( QAction* subAction, subs )
         {
             if( subAction->property( TheStream::CHANNEL_PROPERTY ).toInt() == subId )
@@ -695,13 +716,13 @@ MainWindow::aboutToShowMenu()
                 subAction->setChecked( true );
                 break;
             }
-            debug() << subAction->property( TheStream::CHANNEL_PROPERTY ).toInt() << " not checked.";
+            kDebug() << subAction->property( TheStream::CHANNEL_PROPERTY ).toInt() << " not checked.";
         }
     }
     {
         int audioId = TheStream::audioChannel();
         QList< QAction* > audios = action("audio_channels_menu")->menu()->actions();
-        debug() << "audio #" << audioId << " is going to be checked";
+        kDebug() << "audio #" << audioId << " is going to be checked";
         foreach( QAction* audioAction, audios )
         {
             if( audioAction->property( TheStream::CHANNEL_PROPERTY ).toInt() == audioId )
@@ -775,14 +796,13 @@ MainWindow::updateTitleBarText()
     {
         m_titleLabel->setText( TheStream::prettyTitle() );
     }
-    debug() << "set titles ";
+    kDebug() << "set titles ";
 }
 
 #define CHANNELS_CHANGED( function, actionName ) \
 void \
 MainWindow::function( QList< QAction* > subActions ) \
 { \
-DEBUG_BLOCK \
     if( subActions.size() <= 2 ) \
           action( actionName )->setEnabled( false ); \
     else \
@@ -814,7 +834,7 @@ action( const char *name )
         if( ( actionCollection = ((MainWindow*)mainWindow() )->actionCollection() ) )
             action = actionCollection->action( name );
     if( !action )
-        debug() << name;
+        kDebug() << name;
     Q_ASSERT( mainWindow() );
     Q_ASSERT( actionCollection );
     Q_ASSERT( action );
@@ -823,6 +843,6 @@ action( const char *name )
 }
 
 
-} //namespace Codeine
+} //namespace Dragon
 
 #include "mainWindow.moc"
