@@ -58,21 +58,24 @@
 #include "kmixtoolbox.h"
 #include "version.h"
 #include "viewdockareapopup.h"
-//#include "osd.h" // Postponed to KDE4.1
+#include "dialogselectmaster.h"
+
+#include "osdwidget.h"
 
 
 /* KMixWindow
  * Constructs a mixer window (KMix main window)
  */
 KMixWindow::KMixWindow(bool invisible)
-   : KXmlGuiWindow(0),
+   : KXmlGuiWindow(0, Qt::WindowFlags( KDE_DEFAULT_WINDOWFLAGS | Qt::WindowContextHelpButtonHint) ),
    m_showTicks( true ),
    m_showMenubar(true),
 //   m_isVisible (false),    // initialize, as we don't trigger a hideEvent()
 //   m_visibilityUpdateAllowed( true ),
    m_multiDriverMode (false), // -<- I never-ever want the multi-drivermode to be activated by accident
-   m_dockWidget()
-   , _dockAreaPopup(0)
+   m_dockWidget(),
+   m_dontSetDefaultCardOnStart (false),
+   _dockAreaPopup(0)
 {
     setObjectName("KMixWindow");
     // disable delete-on-close because KMix might just sit in the background waiting for cards to be plugged in
@@ -80,11 +83,12 @@ KMixWindow::KMixWindow(bool invisible)
 
    initActions(); // init actions first, so we can use them in the loadConfig() already
    loadConfig(); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
+   KGlobal::locale()->insertCatalog("kmix-controls");
    initWidgets();
    initPrefDlg();
    MixerToolBox::instance()->initMixer(m_multiDriverMode, m_hwInfoString);
    KMixDeviceManager *theKMixDeviceManager = KMixDeviceManager::instance();
-   recreateGUI();
+   recreateGUI(false);
    fixConfigAfterRead();
    theKMixDeviceManager->initHotplug();
    connect(theKMixDeviceManager, SIGNAL( plugged( const char*, const QString&, QString&)), SLOT (plugged( const char*, const QString&, QString&) ) );
@@ -124,6 +128,9 @@ void KMixWindow::initActions()
    action = actionCollection()->addAction("toggle_channels_currentview");
    action->setText(i18n("Configure &Channels..."));
    connect(action, SIGNAL(triggered(bool) ), SLOT(slotConfigureCurrentView()));
+   action = actionCollection()->addAction( "select_master" );
+   action->setText( i18n("Select Master Channel...") );
+   connect(action, SIGNAL(triggered(bool) ), SLOT(slotSelectMaster()));
 
    KAction* globalAction = actionCollection()->addAction("increase_volume");
    globalAction->setText(i18n("Increase Volume"));
@@ -140,18 +147,7 @@ void KMixWindow::initActions()
    globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeMute));
    connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotMute()));
 
-   volumeDisplay = new QProgressBar();
-   volumeDisplay->setWindowFlags(Qt::X11BypassWindowManagerHint);
-   QDesktopWidget* desktop = KApplication::kApplication()->desktop();
-   //Getting QRect of the screen where cursor is positioned
-   QRect rect = desktop->screenGeometry(QCursor::pos());
-   int width = (rect.width()/2) - (volumeDisplay->width()/2);
-   int height = (rect.height()/2) - (volumeDisplay->height()/2);
-   width += rect.x();
-   height += rect.y();
-   volumeDisplay->move(width, height);
-   volumeDisplayTimer = new QTimer(this);
-   connect(volumeDisplayTimer, SIGNAL(timeout()), this, SLOT(slotHideVolumeDisplay()));
+   osdWidget = new OSDWidget();
 
    createGUI( "kmixui.rc" );
 }
@@ -214,18 +210,23 @@ bool KMixWindow::updateDocking()
       return false;
    }
 
-    // create dock widget and the corresponding popup
-    /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
+   // create dock widget and the corresponding popup
+   /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
    QWidget* referenceWidgetForSystray = this;
-     if ( m_volumeWidget ) {
-      _dockAreaPopup = new ViewDockAreaPopup(0, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
+   if ( m_volumeWidget ) {
+      KMenu *volMenu = new KMenu(this);
+      _dockAreaPopup = new ViewDockAreaPopup(volMenu, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
       _dockAreaPopup->createDeviceWidgets();
-      referenceWidgetForSystray = _dockAreaPopup;
-    }
-    m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
-    m_dockWidget->show();
-    connect(m_dockWidget, SIGNAL(newMasterSelected()), SLOT(saveConfig()) );
-    return true;
+
+      QWidgetAction *volWA = new QWidgetAction(volMenu);
+      volWA->setDefaultWidget(_dockAreaPopup);
+      volMenu->addAction(volWA);
+      referenceWidgetForSystray = volMenu;
+   }
+   m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
+   //m_dockWidget->show();
+   connect(m_dockWidget, SIGNAL(newMasterSelected()), SLOT(saveConfig()) );
+   return true;
 }
 
 void KMixWindow::saveConfig()
@@ -267,7 +268,7 @@ void KMixWindow::saveBaseConfig()
    }
    QString mixerIgnoreExpression = MixerToolBox::instance()->mixerIgnoreExpression();
    config.writeEntry( "MixerIgnoreExpression", mixerIgnoreExpression );
-   
+
    // @todo basically this should be moved in the views later (e.g. KDE4.2 ?)
    if ( m_toplevelOrientation  == Qt::Horizontal )
       config.writeEntry( "Orientation","Horizontal" );
@@ -387,10 +388,16 @@ KMixWindow::loadVolumes()
 
 
 
+void KMixWindow::recreateGUIwithoutSavingView()
+{
+	recreateGUI(false);
+}
+
+
 /**
  * Create or recreate the Mixer GUI elements
  */
-void KMixWindow::recreateGUI()
+void KMixWindow::recreateGUI(bool saveConfig)
 {
    saveViewConfig();  // save the state before recreating
    clearMixerWidgets();
@@ -442,7 +449,7 @@ void KMixWindow::plugged( const char* driverName, const QString& /*udi*/, QStrin
     if ( mixer != 0 ) {
         kDebug(67100) << "Plugged: dev=" << dev << "\n";
         MixerToolBox::instance()->possiblyAddMixer(mixer);
-        recreateGUI();
+        recreateGUI(true);
     }
 
 // Test code for OSD. But OSD is postponed to KDE4.1
@@ -484,7 +491,7 @@ void KMixWindow::unplugged( const QString& udi)
                     Mixer::setGlobalMaster( ((Mixer::mixers())[0])->id(), localMaster);
                     
                     QString text;
-                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2", 
+                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2.", 
                             ((Mixer::mixers())[0])->getLocalMasterMD()->readableName(),
                             ((Mixer::mixers())[0])->readableName()
                                 );
@@ -496,7 +503,7 @@ void KMixWindow::unplugged( const QString& udi)
                 text = i18n("The last soundcard was unplugged.");
                 KMixToolBox::notification("MasterFallback", text);
             }
-            recreateGUI();
+            recreateGUI(true);
             break;
         }
     }
@@ -608,7 +615,9 @@ void KMixWindow::hideOrClose ( )
 void KMixWindow::increaseOrDecreaseVolume(bool increase)
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer(); // only needed for the awkward construct below
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
   md->setMuted(false);
   if (increase)
     mixer->increaseVolume(md->id());    // this is awkward. Better move the increaseVolume impl to the Volume class.
@@ -631,38 +640,30 @@ void KMixWindow::slotDecreaseVolume()
 void KMixWindow::showVolumeDisplay()
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
   int currentVolume = mixer->volume(md->id());
-  if (md->isMuted()) {
-    currentVolume = 0;
-  }
-  volumeDisplay->setValue(currentVolume);
-  volumeDisplay->show();
+  
+  osdWidget->setCurrentVolume(currentVolume, md->isMuted());
+  osdWidget->show();
+  osdWidget->activateOSD(); //Enable the hide timer
 
-  //FIXME, how to get this to work before it is displayed for the first time?
-  QDesktopWidget* desktop = KApplication::kApplication()->desktop();
-  //Getting QRect of the screen where cursor is positioned
-  QRect rect = desktop->screenGeometry(QCursor::pos());
-  int width = (rect.width()/2) - (volumeDisplay->width()/2);
-  int height = (rect.height()/2) - (volumeDisplay->height()/2);
-  width += rect.x();
-  height += rect.y();
-  volumeDisplay->move(width, height);
-
-  volumeDisplayTimer->setInterval(2000);
-  volumeDisplayTimer->start();
-}
-
-void KMixWindow::slotHideVolumeDisplay()
-{
-  volumeDisplay->hide();
+  //Center the OSD
+  QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
+  QSize size = osdWidget->sizeHint();
+  int posX = rect.x() + (rect.width() - size.width()) / 2;
+  int posY = rect.y() + 4 * rect.height() / 5;
+  osdWidget->setGeometry(posX, posY, size.width(), size.height());
 }
 
 void KMixWindow::slotMute()
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
-  mixer->toggleMute(md->id());
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
+  mixer->toggleMute(md->id()); 
   showVolumeDisplay();
 }
 
@@ -731,7 +732,7 @@ void KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    }
 
    if ( labelsHasChanged || ticksHasChanged || dockwidgetHasChanged || toplevelOrientationHasChanged || systrayPopupHasChanged) {
-      recreateGUI();
+      recreateGUI(true);
    }
 
    this->repaint(); // make KMix look fast (saveConfig() often uses several seconds)
@@ -779,6 +780,12 @@ void KMixWindow::slotConfigureCurrentView()
     if (view) view->configureView();
 }
 
+void KMixWindow::slotSelectMaster()
+{
+   DialogSelectMaster* dsm = new DialogSelectMaster(Mixer::getGlobalMasterMixer());
+   if (dsm) dsm->show();
+}
+
 void KMixWindow::newMixerShown(int /*tabIndex*/ ) {
    KMixerWidget* mw = (KMixerWidget*)m_wsMixers->currentWidget();
    if (mw) {
