@@ -463,8 +463,8 @@ void MDWSlider::addSliders( QBoxLayout *volLayout, char type, bool addLabel)
         label->installEventFilter( this );
    }
 
-    for( int i = 0; i < vol.count(); i++ )
-    {
+    for ( int i=0; i<= Volume::CHIDMAX; i++ ) {
+     if ( vol._chmask & Volume::_channelMaskEnum[i] ) {
         Volume::ChannelID chid = Volume::ChannelID(i);
 
         long minvol = vol.minVolume();
@@ -508,6 +508,7 @@ void MDWSlider::addSliders( QBoxLayout *volLayout, char type, bool addLabel)
         ref_sliders.append ( slider );   // add to list
         ref_slidersChids.append(chid);
         connect( slider, SIGNAL(valueChanged(int)), SLOT(volumeChange(int)) );
+     } //if channel is present
     } // for all channels of this device
 }
 
@@ -515,49 +516,60 @@ void MDWSlider::addSliders( QBoxLayout *volLayout, char type, bool addLabel)
 QPixmap MDWSlider::icon( int icontype )
 {
    QPixmap miniDevPM;
+
    switch (icontype) {
       case MixDevice::AUDIO:
-         miniDevPM = loadIcon( "mix_audio"); break;
+         miniDevPM = _iconName = "mixer-pcm"; break;
       case MixDevice::BASS:
       case MixDevice::SURROUND_LFE:  // "LFE" SHOULD have an own icon
-          miniDevPM = loadIcon("mix_bass"); break;
+          miniDevPM = _iconName ="mixer-lfe"; break;
       case MixDevice::CD:
-          miniDevPM = loadIcon("mix_cd"); break;
+          miniDevPM = _iconName ="mixer-cd"; break;
       case MixDevice::EXTERNAL:
-         miniDevPM = loadIcon( "audio-input-line"); break;
+         miniDevPM = _iconName = "mixer-line"; break;
       case MixDevice::MICROPHONE:
-          miniDevPM = loadIcon("audio-input-microphone");break;
+          miniDevPM = _iconName ="mixer-microphone";break;
       case MixDevice::MIDI:
-          miniDevPM = loadIcon("mix_midi"); break;
+          miniDevPM = _iconName ="mixer-midi"; break;
       case MixDevice::RECMONITOR:
-          miniDevPM = loadIcon("mix_recmon"); break;
+          miniDevPM = _iconName ="mixer-capture"; break;
       case MixDevice::TREBLE:
-          miniDevPM = loadIcon("mix_treble"); break;
+          miniDevPM = _iconName ="mixer-pcm-default"; break;
       case MixDevice::UNKNOWN:
-          miniDevPM = loadIcon("mix_unknown"); break;
+          miniDevPM = _iconName ="mixer-front"; break;
       case MixDevice::VOLUME:
-          miniDevPM = loadIcon("mix_volume"); break;
+          miniDevPM = _iconName ="mixer-master"; break;
       case MixDevice::VIDEO:
-          miniDevPM = loadIcon("mix_video"); break;
+          miniDevPM = _iconName ="mixer-video"; break;
       case MixDevice::SURROUND:
       case MixDevice::SURROUND_BACK:
+          miniDevPM = _iconName = "mixer-surround"; break;
       case MixDevice::SURROUND_CENTERFRONT:
       case MixDevice::SURROUND_CENTERBACK:
-          miniDevPM = loadIcon("mix_surround"); break;
+          miniDevPM = _iconName ="mixer-surround-center"; break;
       case MixDevice::HEADPHONE:
-         miniDevPM = loadIcon( "audio-headset" ); break;
+         miniDevPM = _iconName = "mixer-headset"; break;
       case MixDevice::DIGITAL:
-          miniDevPM = loadIcon( "mix_digital" ); break;
+          miniDevPM = _iconName = "mixer-digital"; break;
       case MixDevice::AC97:
-          miniDevPM = loadIcon( "mix_ac97" ); break;
+          miniDevPM = _iconName = "mixer-ac97"; break;
+      case MixDevice::SPEAKER:
+          miniDevPM = _iconName = "mixer-pc-speaker"; break;
+      case MixDevice::MICROPHONE_BOOST:
+          miniDevPM = _iconName = "mixer-microphone-boost"; break;
+      case MixDevice::MICROPHONE_FRONT_BOOST:
+          miniDevPM = _iconName = "mixer-microphone-front-boost"; break;
+      case MixDevice::MICROPHONE_FRONT:
+          miniDevPM = _iconName = "mixer-microphone-front"; break;
       default:
-          miniDevPM = loadIcon("mix_unknown"); break;
+          miniDevPM = _iconName ="mixer-front"; break;
    }
 
+   miniDevPM = loadIcon(_iconName);
    return miniDevPM;
 }
 
-QPixmap MDWSlider::loadIcon( const char*  filename )
+QPixmap MDWSlider::loadIcon( QString& filename )
 {
     return  KIconLoader::global()->loadIcon( filename, KIconLoader::Small, KIconLoader::SizeSmallMedium );
 }
